use axum::{extract::Query, routing::get, Router};
use cairo::{Context, Format, ImageSurface};
use serde::Deserialize;

use crate::{polygon, SharedState};

fn default_as_false() -> bool {
    false
}

#[derive(Deserialize)]
struct ImageProperties {
    #[serde(default = "default_as_false")]
    dark_mode: bool,
}

async fn handler(Query(properties): Query<ImageProperties>) -> impl axum::response::IntoResponse {
    let surface = ImageSurface::create(Format::ARgb32, 400, 400).unwrap();
    let context = Context::new(&surface).unwrap();

    if properties.dark_mode {
        context.set_source_rgb(0.0, 0.0, 0.0);
    } else {
        context.set_source_rgb(1.0, 1.0, 1.0);
    }

    context.paint().unwrap();

    polygon::draw_polygon_of_segmented_polygons(
        (200.0, 200.0),
        200.0,
        6,
        &context,
        properties.dark_mode,
    )
    .unwrap();

    let mut data: Vec<u8> = Vec::new();
    surface.write_to_png(&mut data).unwrap();

    (
        axum::response::AppendHeaders([(axum::http::header::CONTENT_TYPE, "image/png")]),
        data,
    )
}

pub fn routes() -> Router<SharedState> {
    Router::new().route("/", get(handler))
}
